"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const _ = __importStar(require("lodash"));
const os_1 = require("os");
const path = __importStar(require("path"));
const consts_1 = require("./consts");
const errors_1 = require("./errors");
const http_serializer_1 = require("./http-serializer");
const debug = require('debug')('yesno:mocks');
/**
 * Read mocks from a specified file.
 *
 * @throws YesNoError If file is improperly formatted
 */
async function load({ filename }) {
    debug('Loading mocks from', filename);
    let data;
    try {
        data = await fs_extra_1.readFile(filename);
    }
    catch (e) {
        if (e.code === 'ENOENT') {
            throw new errors_1.YesNoError(`${helpMessageMissingMock(filename)}${os_1.EOL}${os_1.EOL}Original error: ${e.message}`);
        }
        throw e;
    }
    let obj;
    const dataString = data.toString();
    try {
        obj = JSON.parse(dataString);
    }
    catch (e) {
        throw new errors_1.YesNoError(`Failed to parse JSON from ${filename}: ${e}`);
    }
    if (!obj.records) {
        throw new errors_1.YesNoError('Invalid JSON format. Missing top level "records" key.');
    }
    return obj.records;
}
exports.load = load;
/**
 * Save HTTP records to the specified file
 */
async function save({ filename, records = [], }) {
    debug('Saving %d records to %s', records.length, filename);
    await fs_extra_1.ensureDir(path.dirname(filename));
    const payload = { records };
    const contents = JSON.stringify(payload, null, 2);
    await fs_extra_1.writeFile(filename, contents);
    return filename;
}
exports.save = save;
function helpMessageMissingMock(filename) {
    const { name } = path.parse(filename);
    // tslint:disable-next-line:max-line-length
    return `Mock file for "${name}" does not exist. To generate the missing file now you may run the command:${os_1.EOL}${os_1.EOL}./node_modules/.bin/yesno generate "${filename}"`;
}
function hydrateHttpMock(mock) {
    const { request, response } = mock;
    let responseHeaders = response.headers || {};
    const responseBody = response.body || '';
    if (_.isPlainObject(response.body) && !responseHeaders[consts_1.HEADER_CONTENT_TYPE]) {
        debug('Adding missing header %s=%s', consts_1.HEADER_CONTENT_TYPE, consts_1.MIME_TYPE_JSON);
        responseHeaders = Object.assign({}, responseHeaders, { [consts_1.HEADER_CONTENT_TYPE]: consts_1.MIME_TYPE_JSON });
    }
    return http_serializer_1.createRecord({
        duration: 0,
        request: Object.assign({ 
            // supply defaults for headers, path, and port
            headers: {}, path: '/', port: request.protocol === 'https' ? consts_1.DEFAULT_PORT_HTTPS : consts_1.DEFAULT_PORT_HTTP }, request),
        response: {
            body: responseBody,
            headers: responseHeaders,
            statusCode: response.statusCode,
        },
    });
}
exports.hydrateHttpMock = hydrateHttpMock;
/**
 * Get the generated filename for a mock name.
 */
function getMockFilename(name, dir) {
    return path.join(dir, `${name.replace(/\s+/g, '-').toLowerCase()}-yesno.json`);
}
exports.getMockFilename = getMockFilename;
//# sourceMappingURL=file.js.map