"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const os_1 = require("os");
const consts_1 = require("../consts");
const errors_1 = require("../errors");
const http_serializer_1 = require("../http-serializer");
const redact_1 = require("./redact");
/**
 * Represents a collection of HTTP requests which match the provided filter.
 *
 * Can filter both intercepted HTTP requests and loaded mocks.
 */
class FilteredHttpCollection {
    constructor({ context, matcher = {} }) {
        this.ctx = context;
        this.matcher = matcher;
    }
    /**
     * Return all intercepted requests matching the current filter
     */
    intercepted() {
        return this.ctx.getMatchingIntercepted(this.matcher);
    }
    /**
     * Return all intercepted mocks matching the current filter
     */
    mocks() {
        return this.ctx.getMatchingMocks(this.matcher);
    }
    /**
     * Provide a mock response for all matching requests.
     *
     * Use callback to dynamically generate response per request.
     *
     * Matching responses defined here take _precedence_ over mocks loaded normally.
     * @param response Serialized HTTP response or callback
     */
    respond(response) {
        this.ctx.addResponseForMatchingRequests({ response, matcher: this.matcher });
    }
    /**
     * Redact given property/properties on all intercepted requests matching current filter.
     *
     * Useful whenever your HTTP requests contain sensitive details such as an API key that should not
     * be saved to disc.
     * @param property Nested path(s) for properties to redact from intercepted HTTP requests.
     *  Works for all properties on serialized HTTP request/response objects.
     *  Accepts dot notation for nested properties (eg `response.body.foobars[0].id`)
     * @param redactor Custom redactor. Defaults to replacing matching values with "*****"
     */
    redact(property, redactor = () => consts_1.DEFAULT_REDACT_SYMBOL) {
        const properties = _.isArray(property) ? property : [property];
        const redactedRecords = this.intercepted().map((intercepted) => redact_1.redact(intercepted, properties, redactor));
        redactedRecords.forEach((redactedRecord) => {
            const i = this.ctx.interceptedRequestsCompleted.findIndex((req) => {
                return redactedRecord.__id === req.__id;
            });
            this.ctx.interceptedRequestsCompleted[i] = redactedRecord;
        });
    }
    /**
     * Return serialized request part of the _single_ matching intercepted HTTP request.
     *
     * Throws an exception if multiple requests were matched.
     */
    request() {
        return this.only().request;
    }
    /**
     * Return serialized response part of the _single_ matching intercepted HTTP request.
     *
     * Throws an exception if multiple requests were matched.
     */
    response() {
        return this.only().response;
    }
    /**
     * If the current filter only matches a single request, then return the single matching instance.
     * Otherwise, throw an error.
     */
    only() {
        const intercepted = this.intercepted();
        const all = this.ctx.interceptedRequestsCompleted;
        if (!intercepted.length) {
            const nonMatchingHint = all
                .map(({ request }, i) => `  ${i + 1}. ${request.method} ${http_serializer_1.formatUrl(request)}`)
                .slice(0, 6)
                .join(os_1.EOL);
            const queryHint = _.isObject(this.matcher)
                ? JSON.stringify(this.matcher, (key, value) => (value instanceof RegExp ? `RegExp(${value.toString()})` : value), 2)
                : 'Function';
            const numNotShown = this.ctx.interceptedRequestsCompleted.length > 6
                ? this.ctx.interceptedRequestsCompleted.length - 6
                : 0;
            // tslint:disable-next-line:max-line-length
            const help = `  ${os_1.EOL}Query:${os_1.EOL}${queryHint}${os_1.EOL}${os_1.EOL}Non-matching intercepted:${os_1.EOL}${nonMatchingHint}${numNotShown ? ` (+${numNotShown} more)` : ''}`;
            throw new errors_1.YesNoError(`No matching intercepted requests ${help}`);
        }
        if (intercepted.length > 1) {
            throw new errors_1.YesNoError(`Query unexpectedly matched multiple (${intercepted.length}) requests.`);
        }
        return intercepted[0];
    }
}
exports.default = FilteredHttpCollection;
//# sourceMappingURL=collection.js.map