"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const t = __importStar(require("io-ts"));
const io_ts_reporters_1 = require("io-ts-reporters");
const _ = __importStar(require("lodash"));
const stream_1 = require("stream");
const uuid = require("uuid");
const consts_1 = require("./consts");
const errors_1 = require("./errors");
const debug = require('debug')('yesno:http-serializer');
const SCHEMA_VERSION = '1.0.0';
/* tslint:disable:max-classes-per-file */
// @see http.OutgoingHttpHeaders
const Headers = t.dictionary(t.string, t.union([t.number, t.string, t.array(t.string), t.undefined]));
const SerializedRequestOptional = t.partial({
    body: t.readonly(t.union([t.string, t.object])),
    query: t.readonly(t.string),
});
const SerializedRequest = t.intersection([
    t.interface({
        headers: t.readonly(Headers),
        host: t.readonly(t.string),
        method: t.readonly(t.string),
        path: t.readonly(t.string),
        port: t.readonly(t.Integer),
        protocol: t.readonly(t.union([t.literal('http'), t.literal('https')])),
    }),
    SerializedRequestOptional,
]);
const SerializedResponse = t.interface({
    body: t.readonly(t.union([t.string, t.object])),
    headers: t.readonly(Headers),
    statusCode: t.readonly(t.Integer),
});
const SerializedHttpOptional = t.partial({
    __duration: t.readonly(t.number),
    __timestamp: t.readonly(t.number),
});
const SerializedHttp = t.intersection([
    SerializedHttpOptional,
    t.interface({
        __id: t.readonly(t.string),
        __version: t.readonly(t.string),
        request: SerializedRequest,
        response: SerializedResponse,
    }),
]);
function serializeJSON(headers, body) {
    const isJSON = headers[consts_1.HEADER_CONTENT_TYPE] &&
        headers[consts_1.HEADER_CONTENT_TYPE].includes(consts_1.MIME_TYPE_JSON);
    if (isJSON) {
        try {
            body = JSON.parse(body);
        }
        catch (e) {
            // Don't throw, just log and continue with unparsed body
            debug('Failed to parse JSON body', body);
        }
    }
    return body;
}
class RequestSerializer extends stream_1.Transform {
    constructor(originalClientOpts, originalClientReq, interceptedServerReq, isHttps) {
        super();
        this.headers = {};
        // @see https://github.com/nodejs/node/blob/v10.11.0/lib/_http_client.js#L121
        const agent = originalClientReq.agent;
        const defaultPort = originalClientOpts.defaultPort || (agent && agent.defaultPort);
        const port = originalClientOpts.port || defaultPort || 80;
        this.port = typeof port === 'string' ? parseInt(port, 10) : port;
        // @see https://github.com/nodejs/node/blob/v10.11.0/lib/_http_client.js#L125
        const [path, query] = originalClientReq.path.split('?');
        this.host = originalClientOpts.hostname || originalClientOpts.host || 'localhost';
        this.method = interceptedServerReq.method.toUpperCase();
        this.path = path;
        this.query = query ? `?${query}` : query;
        this.protocol = isHttps ? 'https' : 'http';
        this.headers = _.omit(originalClientReq.getHeaders(), consts_1.YESNO_INTERNAL_HTTP_HEADER);
    }
    _transform(chunk, encoding, done) {
        this.body = this.body || '';
        this.body += chunk.toString(); // @todo Do we really need to convert to string for each chunk?
        this.push(chunk);
        done();
    }
    serialize() {
        return {
            body: serializeJSON(this.headers, this.body),
            headers: this.headers,
            host: this.host,
            method: this.method,
            path: this.path,
            port: this.port,
            protocol: this.protocol,
            query: this.query,
        };
    }
}
exports.RequestSerializer = RequestSerializer;
class ResponseSerializer extends stream_1.Transform {
    constructor(clientResponse) {
        super();
        this.headers = {};
        this.body = '';
        this.statusCode = clientResponse.statusCode;
        this.headers = clientResponse.headers;
    }
    _transform(chunk, encoding, done) {
        this.body += chunk.toString();
        this.push(chunk);
        done();
    }
    serialize() {
        return {
            body: serializeJSON(this.headers, this.body),
            headers: this.headers,
            statusCode: this.statusCode,
        };
    }
}
exports.ResponseSerializer = ResponseSerializer;
function formatUrl(request, includePort = false) {
    const port = includePort ? `:${request.port}` : '';
    const query = request.query || '';
    return `${request.protocol}://${request.host}${port}${request.path}${query}`;
}
exports.formatUrl = formatUrl;
/**
 * Create record for an HTTP request, which may be saved in a mock file.
 */
function createRecord({ request, response, duration }) {
    return {
        __duration: duration,
        __id: uuid.v4(),
        __timestamp: Date.now(),
        __version: SCHEMA_VERSION,
        request,
        response,
    };
}
exports.createRecord = createRecord;
function validateSerializedHttpArray(records) {
    const result = t.array(SerializedHttp).decode(records);
    if (result.isLeft()) {
        const errs = io_ts_reporters_1.reporter(result);
        throw new errors_1.YesNoError(`Invalid serialized HTTP. (Errors: ${errs.join(', ')})`);
    }
}
exports.validateSerializedHttpArray = validateSerializedHttpArray;
//# sourceMappingURL=http-serializer.js.map