"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const http = __importStar(require("http"));
const http_1 = require("http");
const https = __importStar(require("https"));
const _ = __importStar(require("lodash"));
const mitm_1 = __importDefault(require("mitm"));
const readable = __importStar(require("readable-stream"));
const uuid_1 = require("uuid");
const consts_1 = require("./consts");
const http_serializer_1 = require("./http-serializer");
const debug = require('debug')('yesno:proxy');
/**
 * Intercept outbound HTTP requests and provide mock responses through an event API.
 *
 * Uses MITM library to spy on HTTP requests made in current NodeJS process.
 */
class Interceptor extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        this.requestNumber = 0;
        this.clientRequests = {};
        this.ignorePorts = [];
    }
    /**
     * Enables intercepting all outbound HTTP requests.
     * @param options Intercept options
     */
    enable(options = {}) {
        if (this.mitm || this.origOnSocket) {
            debug('Interceptor already enabled. Do nothing.');
            return;
        }
        const self = this;
        this.mitm = mitm_1.default();
        this.ignorePorts = options.ignorePorts || [];
        this.origOnSocket = http_1.ClientRequest.prototype.onSocket;
        http_1.ClientRequest.prototype.onSocket = _.flowRight(this.origOnSocket, 
        // tslint:disable-next-line:only-arrow-functions
        function (socket) {
            if (undefined !== socket.__yesno_req_id) {
                debug('New socket on client request', socket.__yesno_req_id);
                self.clientRequests[socket.__yesno_req_id].clientRequest = this;
                this.setHeader(consts_1.YESNO_INTERNAL_HTTP_HEADER, socket.__yesno_req_id);
            }
            return socket;
        });
        this.mitm.on('connect', this.mitmOnConnect.bind(this));
        this.mitm.on('request', this.mitmOnRequest.bind(this));
        this.mitm.on('connection', (server) => {
            server.on('error', (err) => debug('Server error:', err));
        });
    }
    /**
     * Disables intercepting outbound HTTP requests.
     */
    disable() {
        if (!this.mitm || !this.origOnSocket) {
            debug('Interceptor already disabled. Do nothing.');
            return;
        }
        http_1.ClientRequest.prototype.onSocket = this.origOnSocket;
        this.mitm.disable();
        this.mitm = undefined;
        this.origOnSocket = undefined;
    }
    /**
     * Event handler for Mitm "connect" event.
     */
    mitmOnConnect(socket, options) {
        debug('New socket connection');
        const { port } = options;
        if (this.ignorePorts && -1 !== this.ignorePorts.indexOf(parseInt(String(port), 10))) {
            debug('Ignoring socket on port %d', port);
            socket.bypass();
            return;
        }
        if (options.proxying) {
            debug('Bypassing intercept...');
            socket.bypass();
        }
        else {
            this.trackSocketAndClientOptions(socket, options);
            debug('Tracking socket %s', socket.__yesno_req_id);
        }
    }
    /**
     * Event handler for Mitm "request" event.
     *
     * Intercepted requests will be proxied if the `shouldProxy` option has been set.
     * @emits 'intercept' when we intercept a request
     * @emits 'proxied' when we have sent the response for a proxied response
     */
    mitmOnRequest(interceptedRequest, interceptedResponse) {
        debug('mitm event:request');
        const id = this.getRequestId(interceptedRequest);
        function debugReq(formatter, ...args) {
            debug(`[id:${id}] ${formatter}`, ...args);
        }
        if (!this.clientRequests[id]) {
            debugReq(`Error: Missing client options for yesno req ${id}`);
            throw new Error(`Missing client options for yesno req ${id}`);
        }
        interceptedResponse.on('finish', () => {
            debugReq('Response finished');
        });
        const { clientOptions } = this.clientRequests[id];
        const clientRequest = this.clientRequests[id].clientRequest;
        const isHttps = interceptedRequest.connection.encrypted;
        const requestSerializer = new http_serializer_1.RequestSerializer(clientOptions, clientRequest, interceptedRequest, isHttps);
        const requestNumber = this.requestNumber;
        this.requestNumber++;
        debugReq('Emitting "intercept"');
        const proxy = () => {
            const request = isHttps ? https.request : http.request;
            const proxiedRequest = request(Object.assign({}, clientOptions, { headers: _.omit(clientRequest.getHeaders(), consts_1.YESNO_INTERNAL_HTTP_HEADER), path: clientRequest.path, proxying: true }));
            readable.pipeline(interceptedRequest, requestSerializer, proxiedRequest);
            interceptedRequest.on('error', (e) => debugReq('Error on intercepted request:', e));
            interceptedRequest.on('aborted', () => {
                debugReq('Intercepted request aborted');
                proxiedRequest.abort();
            });
            proxiedRequest.on('timeout', (e) => debugReq('Proxied request timeout', e));
            proxiedRequest.on('error', (e) => debugReq('Proxied request error', e));
            proxiedRequest.on('aborted', () => debugReq('Proxied request aborted'));
            proxiedRequest.on('response', (proxiedResponse) => {
                const responseSerializer = new http_serializer_1.ResponseSerializer(proxiedResponse);
                debugReq('proxied response (%d)', proxiedResponse.statusCode);
                if (proxiedResponse.statusCode) {
                    interceptedResponse.writeHead(proxiedResponse.statusCode, proxiedResponse.headers);
                }
                readable.pipeline(proxiedResponse, responseSerializer, interceptedResponse);
                proxiedResponse.on('end', () => {
                    debugReq('Emitting "proxied"');
                    this.emit('proxied', {
                        requestNumber,
                        requestSerializer,
                        responseSerializer,
                    });
                });
            });
        };
        // YesNo will listen for this event to mock the response
        this.emit('intercept', {
            clientRequest,
            interceptedRequest,
            interceptedResponse,
            proxy,
            requestNumber,
            requestSerializer,
        });
    }
    trackSocketAndClientOptions(socket, clientOptions) {
        socket.__yesno_req_id = uuid_1.v4();
        this.clientRequests[socket.__yesno_req_id] = { clientOptions };
    }
    getRequestId(interceptedRequest) {
        const id = interceptedRequest.headers[consts_1.YESNO_INTERNAL_HTTP_HEADER];
        if (!id) {
            throw new Error('Missing internal header');
        }
        return id;
    }
}
exports.default = Interceptor;
//# sourceMappingURL=interceptor.js.map