"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const readable = __importStar(require("readable-stream"));
const errors_1 = require("./errors");
const recording_1 = require("./recording");
const debug = require('debug')('yesno:mock-response');
/**
 * Encapsulates logic for sending a response for an intercepted HTTP request
 */
class MockResponse {
    constructor(event, ctx) {
        this.ctx = ctx;
        this.event = event;
    }
    /**
     * Send a respond for our wrapped intercept event if able.
     *
     * Give precedence to matching responses in shared context (from `yesno.matching().respond()`).
     * Else, if we're in mock mode, lookup the mock response.
     *
     * @returns The received request & sent response. Returns `undefined` if unable to respond
     */
    async send() {
        const { interceptedRequest, interceptedResponse, requestSerializer, requestNumber, } = this.event;
        let response;
        debug(`[#${requestNumber}] Mock response`);
        await readable.pipeline(interceptedRequest, requestSerializer);
        const request = requestSerializer.serialize();
        response = this.ctx.getResponseDefinedMatching(request);
        if (!response && this.ctx.mode === recording_1.RecordMode.Mock) {
            const mock = this.getMockForIntecept(this.event);
            // Assertion must happen before promise -
            // mitm does not support promise rejections on "request" event
            this.assertMockMatches({ mock, serializedRequest: request, requestNumber });
            response = mock.response;
        }
        if (response) {
            this.writeMockResponse(response, interceptedResponse);
            return { request, response };
        }
    }
    getMockForIntecept({ requestNumber }) {
        const mock = this.ctx.loadedMocks[requestNumber];
        if (!mock) {
            throw new errors_1.YesNoError('No mock found for request');
        }
        return mock;
    }
    assertMockMatches({ mock, serializedRequest, requestNumber, }) {
        try {
            // determine how we'll compare the request and the mock
            const compareBy = this.ctx.comparatorFn;
            // the comparison function must throw an error to signal a mismatch
            compareBy(serializedRequest, mock.request, { requestIndex: requestNumber });
        }
        catch (err) {
            // ensure any user-thrown error is wrapped in our YesNoError
            throw new errors_1.YesNoError(err.message);
        }
    }
    writeMockResponse(response, interceptedResponse) {
        const bodyString = _.isPlainObject(response.body)
            ? JSON.stringify(response.body)
            : response.body;
        const responseHeaders = Object.assign({}, response.headers);
        if (responseHeaders['content-length'] &&
            parseInt(responseHeaders['content-length'], 10) !== Buffer.byteLength(bodyString)) {
            responseHeaders['content-length'] = Buffer.byteLength(bodyString);
        }
        interceptedResponse.writeHead(response.statusCode, responseHeaders);
        interceptedResponse.write(bodyString);
        interceptedResponse.end();
    }
}
exports.default = MockResponse;
//# sourceMappingURL=mock-response.js.map