"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const os_1 = require("os");
const consts_1 = require("./consts");
const errors_1 = require("./errors");
const file = __importStar(require("./file"));
const collection_1 = __importDefault(require("./filtering/collection"));
const http_serializer_1 = require("./http-serializer");
const interceptor_1 = __importDefault(require("./interceptor"));
const mock_response_1 = __importDefault(require("./mock-response"));
const recording_1 = __importStar(require("./recording"));
const debug = require('debug')('yesno');
/**
 * Client API for YesNo
 */
class YesNo {
    constructor(ctx) {
        this.ctx = ctx;
        this.interceptor = this.createInterceptor();
    }
    /**
     * Restore HTTP functionality
     */
    restore() {
        debug('Disabling intercept');
        this.clear();
        this.interceptor.disable();
    }
    /**
     * Spy on intercepted requests
     */
    spy(options) {
        this.enable(options);
        this.setMode(recording_1.RecordMode.Spy);
    }
    /**
     * Mock responses for intercepted requests
     * @todo Reset the request counter?
     */
    mock(mocks, options) {
        this.enable(options);
        this.setMode(recording_1.RecordMode.Mock);
        this.setMocks(mocks.map(file.hydrateHttpMock));
    }
    /**
     * Start a new recording.
     *
     * Depending on the configured mode, will either spy on all outbound HTTP requests
     * or return mocks loaded from disc.
     *
     * When done, call the `complete()` on the returned recording
     * to save all intercepted requests to disc if applicable.
     * @param options Where to load/save mocks
     * @returns A new recording.
     */
    async recording(options) {
        const mode = this.getModeByEnv();
        if (mode !== recording_1.RecordMode.Mock) {
            this.spy();
        }
        else {
            this.mock(await this.load(options));
        }
        return new recording_1.default(Object.assign({}, options, { getRecordsToSave: this.getRecordsToSave.bind(this), mode }));
    }
    /**
     * Create a test function that will wrap its provided test in a recording.
     */
    test({ it, test, dir, prefix }) {
        const runTest = test || it;
        if (!runTest) {
            throw new errors_1.YesNoError('Missing "test" or "it" test function');
        }
        return (title, fn) => {
            const filename = file.getMockFilename(prefix ? `${prefix}-${title}` : title, dir);
            return runTest(title, async () => {
                debug('Running test "%s"', title);
                this.restore();
                try {
                    const recording = await this.recording({ filename });
                    await fn();
                    debug('Saving test "%s"', filename);
                    await recording.complete();
                }
                finally {
                    this.restore();
                }
            });
        };
    }
    /**
     * Load request/response mocks from disk
     */
    async load(options) {
        debug('Loading mocks');
        const records = await file.load(options);
        http_serializer_1.validateSerializedHttpArray(records);
        return records;
    }
    /**
     * Save intercepted requests
     *
     * @returns Full filename of saved JSON if generated
     */
    async save(options) {
        options.records = options.records || this.getRecordsToSave();
        return file.save(options);
    }
    /**
     * Clear all stateful information about requests.
     *
     * If used in a test suite, this should be called after each test.
     */
    clear() {
        this.ctx.clear();
        this.interceptor.requestNumber = 0;
    }
    /**
     * Create a filter collection
     * @todo Convert everything to a match fn
     * @param query
     */
    matching(filter) {
        const normalizedFilter = _.isString(filter) || _.isRegExp(filter) ? { url: filter } : filter;
        return this.getCollection(normalizedFilter);
    }
    /**
     * Get all intercepted requests
     */
    intercepted() {
        return this.getCollection().intercepted();
    }
    /**
     * Get all loaded mocks
     */
    mocks() {
        return this.getCollection().mocks();
    }
    /**
     * Redact property on all records
     */
    redact(property, redactor) {
        return this.getCollection().redact(property, redactor);
    }
    getModeByEnv() {
        const env = (process.env[consts_1.YESNO_RECORDING_MODE_ENV_VAR] || recording_1.RecordMode.Mock).toLowerCase();
        if (!Object.values(recording_1.RecordMode).includes(env)) {
            throw new errors_1.YesNoError(
            // tslint:disable-next-line:max-line-length
            `Invalid mode "${env}" set for ${consts_1.YESNO_RECORDING_MODE_ENV_VAR}. Must be one of ${Object.values(recording_1.RecordMode).join(', ')}`);
        }
        return env;
    }
    getRecordsToSave() {
        const inFlightRequests = this.ctx.inFlightRequests.filter((x) => x);
        if (inFlightRequests.length) {
            const urls = inFlightRequests
                .map(({ requestSerializer }) => `${requestSerializer.method}${http_serializer_1.formatUrl(requestSerializer)}`)
                .join(os_1.EOL);
            throw new errors_1.YesNoError(`Cannot save. Still have ${inFlightRequests.length} in flight requests: ${os_1.EOL}${urls}`);
        }
        return this.ctx.interceptedRequestsCompleted;
    }
    /**
     * Enable intercepting requests
     */
    enable(options) {
        const { comparatorFn, ignorePorts = [] } = options || {};
        debug('Enabling intercept. Ignoring ports', ignorePorts);
        this.interceptor.enable({ ignorePorts });
        this.ctx.comparatorFn = comparatorFn || this.ctx.comparatorFn;
        return this;
    }
    setMocks(mocks) {
        http_serializer_1.validateSerializedHttpArray(mocks);
        this.ctx.loadedMocks = mocks;
    }
    /**
     * Determine the current mode
     */
    isMode(mode) {
        return this.ctx.mode === mode;
    }
    createInterceptor() {
        const interceptor = new interceptor_1.default();
        interceptor.on('intercept', this.onIntercept.bind(this));
        interceptor.on('proxied', this.onProxied.bind(this));
        return interceptor;
    }
    async onIntercept(event) {
        this.recordRequest(event.requestSerializer, event.requestNumber);
        if (!this.ctx.hasResponsesDefinedForMatchers() && !this.isMode(recording_1.RecordMode.Mock)) {
            // No need to mock, send event to its original destination
            return event.proxy();
        }
        try {
            const mockResponse = new mock_response_1.default(event, this.ctx);
            const sent = await mockResponse.send();
            if (sent) {
                this.recordResponse(sent.request, sent.response, event.requestNumber);
            }
            else if (this.isMode(recording_1.RecordMode.Mock)) {
                throw new Error('Unexpectedly failed to send mock respond');
            }
        }
        catch (e) {
            if (!(e instanceof errors_1.YesNoError)) {
                debug(`[#${event.requestNumber}] Mock response failed unexpectedly`, e);
                e.message = `YesNo: Mock response failed: ${e.message}`;
            }
            else {
                debug(`[#${event.requestNumber}] Mock response failed`, e.message);
            }
            event.clientRequest.emit('error', e);
        }
    }
    onProxied({ requestSerializer, responseSerializer, requestNumber }) {
        this.recordResponse(requestSerializer.serialize(), responseSerializer.serialize(), requestNumber);
    }
    setMode(mode) {
        this.ctx.mode = mode;
    }
    getCollection(matcher) {
        return new collection_1.default({
            context: this.ctx,
            matcher,
        });
    }
    recordRequest(requestSerializer, requestNumber) {
        this.ctx.inFlightRequests[requestNumber] = {
            requestSerializer,
            startTime: Date.now(),
        };
    }
    recordResponse(request, response, requestNumber) {
        const duration = Date.now() - this.ctx.inFlightRequests[requestNumber].startTime;
        const record = http_serializer_1.createRecord({ request, response, duration });
        this.ctx.interceptedRequestsCompleted[requestNumber] = record;
        this.ctx.inFlightRequests[requestNumber] = null;
        debug('Added request-response for %s %s (duration: %d)', request.method, record.request.host, duration);
    }
}
exports.YesNo = YesNo;
//# sourceMappingURL=yesno.js.map